@@ -14,17 +14,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <dbus/dbus-glib.h>
-
 #include "gsm-client.h"
-#include "gsm-client-glue.h"
+#include "org.gnome.SessionManager.Client.h"
 
 static guint32 client_serial = 1;
 
@@ -36,7 +32,8 @@ struct GsmClientPrivate
         char            *startup_id;
         char            *app_id;
         guint            status;
-        DBusGConnection *connection;
+        GsmExportedClient *skeleton;
+        GDBusConnection *connection;
 };
 
 enum {
@@ -57,37 +54,23 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_ABSTRACT_TYPE (GsmClient, gsm_client, G_TYPE_OBJECT)
 
+#define GSM_CLIENT_DBUS_IFACE "org.gnome.SessionManager.Client"
+
+static const GDBusErrorEntry gsm_client_error_entries[] = {
+        { GSM_CLIENT_ERROR_GENERAL, GSM_CLIENT_DBUS_IFACE ".GeneralError" },
+        { GSM_CLIENT_ERROR_NOT_REGISTERED, GSM_CLIENT_DBUS_IFACE ".NotRegistered" }
+};
+
 GQuark
 gsm_client_error_quark (void)
 {
-        static GQuark ret = 0;
-        if (ret == 0) {
-                ret = g_quark_from_static_string ("gsm_client_error");
-        }
+        static volatile gsize quark_volatile = 0;
 
-        return ret;
-}
-
-#define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
-
-GType
-gsm_client_error_get_type (void)
-{
-        static GType etype = 0;
-
-        if (etype == 0) {
-                static const GEnumValue values[] = {
-                        ENUM_ENTRY (GSM_CLIENT_ERROR_GENERAL, "GeneralError"),
-                        ENUM_ENTRY (GSM_CLIENT_ERROR_NOT_REGISTERED, "NotRegistered"),
-                        { 0, 0, 0 }
-                };
-
-                g_assert (GSM_CLIENT_NUM_ERRORS == G_N_ELEMENTS (values) - 1);
-
-                etype = g_enum_register_static ("GsmClientError", values);
-        }
-
-        return etype;
+        g_dbus_error_register_error_domain ("gsm_client_error",
+                                            &quark_volatile,
+                                            gsm_client_error_entries,
+                                            G_N_ELEMENTS (gsm_client_error_entries));
+        return quark_volatile;
 }
 
 static guint32
@@ -104,22 +87,112 @@ get_next_client_serial (void)
         return serial;
 }
 
+static gboolean
+gsm_client_get_startup_id (GsmExportedClient     *skeleton,
+                           GDBusMethodInvocation *invocation,
+                           GsmClient             *client)
+{
+        gsm_exported_client_complete_get_startup_id (skeleton, invocation, client->priv->startup_id);
+        return TRUE;
+}
+
+static gboolean
+gsm_client_get_app_id (GsmExportedClient     *skeleton,
+                       GDBusMethodInvocation *invocation,
+                       GsmClient             *client)
+{
+        gsm_exported_client_complete_get_app_id (skeleton, invocation, client->priv->app_id);
+        return TRUE;
+}
+
+static gboolean
+gsm_client_get_restart_style_hint (GsmExportedClient     *skeleton,
+                                   GDBusMethodInvocation *invocation,
+                                   GsmClient             *client)
+{
+        guint hint;
+
+        hint = GSM_CLIENT_GET_CLASS (client)->impl_get_restart_style_hint (client);
+        gsm_exported_client_complete_get_restart_style_hint (skeleton, invocation, hint);
+        return TRUE;
+}
+
+static gboolean
+gsm_client_get_status (GsmExportedClient     *skeleton,
+                       GDBusMethodInvocation *invocation,
+                       GsmClient             *client)
+{
+        gsm_exported_client_complete_get_status (skeleton, invocation, client->priv->status);
+        return TRUE;
+}
+
+static gboolean
+gsm_client_get_unix_process_id (GsmExportedClient     *skeleton,
+                                GDBusMethodInvocation *invocation,
+                                GsmClient             *client)
+{
+        guint pid;
+
+        pid = GSM_CLIENT_GET_CLASS (client)->impl_get_unix_process_id (client);
+        gsm_exported_client_complete_get_unix_process_id (skeleton, invocation, pid);
+        return TRUE;
+}
+
+static gboolean
+gsm_client_stop_dbus (GsmExportedClient     *skeleton,
+                      GDBusMethodInvocation *invocation,
+                      GsmClient             *client)
+{
+        GError *error = NULL;
+        gsm_client_stop (client, &error);
+
+        if (error != NULL) {
+                g_dbus_method_invocation_take_error (invocation, error);
+        } else {
+                gsm_exported_client_complete_stop (skeleton, invocation);
+        }
+
+        return TRUE;
+}
+
 static gboolean
 register_client (GsmClient *client)
 {
-        GError *error;
+        GError *error = NULL;
+        GsmExportedClient *skeleton;
 
-        error = NULL;
-        client->priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+        client->priv->connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
         if (client->priv->connection == NULL) {
-                if (error != NULL) {
-                        g_critical ("error getting session bus: %s", error->message);
-                        g_error_free (error);
-                }
+                g_critical ("error getting session bus: %s", error->message);
+                g_error_free (error);
                 return FALSE;
         }
 
-        dbus_g_connection_register_g_object (client->priv->connection, client->priv->id, G_OBJECT (client));
+        skeleton = gsm_exported_client_skeleton_new ();
+        client->priv->skeleton = skeleton;
+        g_debug ("exporting client to object path: %s", client->priv->id);
+        g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (skeleton),
+                                          client->priv->connection,
+                                          client->priv->id, &error);
+
+        if (error != NULL) {
+                g_critical ("error exporting client on session bus: %s", error->message);
+                g_error_free (error);
+                return FALSE;
+        }
+
+        g_signal_connect (skeleton, "handle-get-app-id",
+                          G_CALLBACK (gsm_client_get_app_id), client);
+        g_signal_connect (skeleton, "handle-get-restart-style-hint",
+                          G_CALLBACK (gsm_client_get_restart_style_hint), client);
+        g_signal_connect (skeleton, "handle-get-startup-id",
+                          G_CALLBACK (gsm_client_get_startup_id), client);
+        g_signal_connect (skeleton, "handle-get-status",
+                          G_CALLBACK (gsm_client_get_status), client);
+        g_signal_connect (skeleton, "handle-get-unix-process-id",
+                          G_CALLBACK (gsm_client_get_unix_process_id), client);
+        g_signal_connect (skeleton, "handle-stop",
+                          G_CALLBACK (gsm_client_stop_dbus), client);
 
         return TRUE;
 }
@@ -169,6 +242,14 @@ gsm_client_finalize (GObject *object)
         g_free (client->priv->startup_id);
         g_free (client->priv->app_id);
 
+        if (client->priv->skeleton != NULL) {
+                g_dbus_interface_skeleton_unexport_from_connection (G_DBUS_INTERFACE_SKELETON (client->priv->skeleton),
+                                                                    client->priv->connection);
+                g_clear_object (&client->priv->skeleton);
+        }
+
+        g_clear_object (&client->priv->connection);
+
         G_OBJECT_CLASS (gsm_client_parent_class)->finalize (object);
 }
 
@@ -348,8 +429,6 @@ gsm_client_class_init (GsmClientClass *klass)
                                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
         g_type_class_add_private (klass, sizeof (GsmClientPrivate));
-
-        dbus_g_object_type_install_info (GSM_TYPE_CLIENT, &dbus_glib_gsm_client_object_info);
 }
 
 const char *
@@ -402,66 +481,6 @@ gsm_client_peek_restart_style_hint (GsmClient *client)
         return GSM_CLIENT_GET_CLASS (client)->impl_get_restart_style_hint (client);
 }
 
-gboolean
-gsm_client_get_startup_id (GsmClient *client,
-                           char     **id,
-                           GError   **error)
-{
-        g_return_val_if_fail (GSM_IS_CLIENT (client), FALSE);
-
-        *id = g_strdup (client->priv->startup_id);
-
-        return TRUE;
-}
-
-gboolean
-gsm_client_get_app_id (GsmClient *client,
-                       char     **id,
-                       GError   **error)
-{
-        g_return_val_if_fail (GSM_IS_CLIENT (client), FALSE);
-
-        *id = g_strdup (client->priv->app_id);
-
-        return TRUE;
-}
-
-gboolean
-gsm_client_get_restart_style_hint (GsmClient *client,
-                                   guint     *hint,
-                                   GError   **error)
-{
-        g_return_val_if_fail (GSM_IS_CLIENT (client), FALSE);
-
-        *hint = GSM_CLIENT_GET_CLASS (client)->impl_get_restart_style_hint (client);
-
-        return TRUE;
-}
-
-gboolean
-gsm_client_get_status (GsmClient *client,
-                       guint     *status,
-                       GError   **error)
-{
-        g_return_val_if_fail (GSM_IS_CLIENT (client), FALSE);
-
-        *status = client->priv->status;
-
-        return TRUE;
-}
-
-gboolean
-gsm_client_get_unix_process_id (GsmClient  *client,
-                                guint      *pid,
-                                GError    **error)
-{
-        g_return_val_if_fail (GSM_IS_CLIENT (client), FALSE);
-
-        *pid = GSM_CLIENT_GET_CLASS (client)->impl_get_unix_process_id (client);
-
-        return TRUE;
-}
-
 /**
  * gsm_client_get_app_name:
  * @client: a #GsmClient.
@@ -524,11 +543,12 @@ gsm_client_disconnected (GsmClient *client)
 
 GKeyFile *
 gsm_client_save (GsmClient *client,
+                 GsmApp    *app,
                  GError   **error)
 {
         g_return_val_if_fail (GSM_IS_CLIENT (client), FALSE);
 
-        return GSM_CLIENT_GET_CLASS (client)->impl_save (client, error);
+        return GSM_CLIENT_GET_CLASS (client)->impl_save (client, app, error);
 }
 
 void
