@@ -318,10 +318,12 @@ gsm_systemd_init (GsmSystemd *manager)
                                       0,
                                       G_MAXINT,
                                       NULL,
-                                      NULL);
+                                      &error);
         if (res == NULL) {
                 g_warning ("Could not get session id for session. Check that logind is "
-                           "properly installed and pam_systemd is getting used at login.");
+                           "properly installed and pam_systemd is getting used at login: %s",
+                           error->message);
+                g_error_free (error);
                 return;
         }
 
@@ -462,6 +464,12 @@ gsm_systemd_set_session_idle (GsmSystem *system,
         GsmSystemd *manager = GSM_SYSTEMD (system);
         GDBusConnection *bus;
 
+        if (manager->priv->session_path == NULL) {
+                g_warning ("Could not get session path for session. Check that logind is "
+                           "properly installed and pam_systemd is getting used at login.");
+                return;
+        }
+
         g_debug ("Updating systemd idle status: %d", is_idle);
         bus = g_dbus_proxy_get_connection (manager->priv->sd_proxy);
         g_dbus_connection_call (bus,
@@ -881,6 +889,68 @@ gsm_systemd_complete_shutdown (GsmSystem *system)
         drop_delay_inhibitor (systemd);
 }
 
+static gboolean
+gsm_systemd_is_last_session_for_user (GsmSystem *system)
+{
+        char **sessions = NULL;
+        char *session = NULL;
+        gboolean is_last_session;
+        int ret, i;
+
+        ret = sd_pid_get_session (getpid (), &session);
+
+        if (ret != 0) {
+                return FALSE;
+        }
+
+        ret = sd_uid_get_sessions (getuid (), FALSE, &sessions);
+
+        if (ret <= 0) {
+                free (session);
+                return FALSE;
+        }
+
+        is_last_session = TRUE;
+        for (i = 0; sessions[i]; i++) {
+                char *state = NULL;
+                char *type = NULL;
+
+                if (g_strcmp0 (sessions[i], session) == 0)
+                        continue;
+
+                ret = sd_session_get_state (sessions[i], &state);
+
+                if (ret != 0)
+                        continue;
+
+                if (g_strcmp0 (state, "closing") == 0) {
+                        free (state);
+                        continue;
+                }
+                free (state);
+
+                ret = sd_session_get_type (sessions[i], &type);
+
+                if (ret != 0)
+                        continue;
+
+                if (g_strcmp0 (type, "x11") != 0 &&
+                    g_strcmp0 (type, "wayland") != 0) {
+                        free (type);
+                        continue;
+                }
+
+                is_last_session = FALSE;
+        }
+
+        for (i = 0; sessions[i]; i++)
+                free (sessions[i]);
+        free (sessions);
+        free (session);
+
+        return is_last_session;
+}
+
 static void
 gsm_systemd_system_init (GsmSystemInterface *iface)
 {
@@ -899,6 +969,7 @@ gsm_systemd_system_init (GsmSystemInterface *iface)
         iface->remove_inhibitor = gsm_systemd_remove_inhibitor;
         iface->prepare_shutdown = gsm_systemd_prepare_shutdown;
         iface->complete_shutdown = gsm_systemd_complete_shutdown;
+        iface->is_last_session_for_user = gsm_systemd_is_last_session_for_user;
 }
 
 GsmSystemd *
